#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
"""
Testing suite for the null executable.
"""

import argparse

import pytest
from null.null import ERRORS, Null
from pytest_mock import MockerFixture


@pytest.fixture
def args():
    return argparse.Namespace(
        dump=False, exit_fail=False, exit_random=False, greeting=False, nap=False, print_error=False, verbose=False
    )


def test_print_error(args, caplog):
    args.print_error = True
    Null(args, True).execute()
    assert "ERROR: This is an error." in caplog.text


def test_print_greeting(args, caplog):
    args.greeting = True
    Null(args, True).execute()
    assert "Hello, world!" in caplog.text
    assert "And goodbye, world..." in caplog.text


def test_take_nap(args, caplog, mocker):
    mocker.patch("time.sleep", return_value=None)
    args.nap = True
    Null(args, True).execute()
    assert "Going to sleep..." in caplog.text
    assert "Waking up." in caplog.text


def test_exit_randomly(args, caplog):
    args.exit_random = True
    with pytest.raises(SystemExit) as e:
        Null(args, True).execute()
    assert "Exiting with status code" in caplog.text
    assert e.value.code in ERRORS.keys()


def test_exit_with_failure(args, caplog):
    args.exit_fail = True
    with pytest.raises(SystemExit) as e:
        Null(args, True).execute()
    assert ERRORS[1] in caplog.text
    assert e.value.code == 1
